@@ -15,14 +15,15 @@ couchTests.reduce = function(debug) {
   db.deleteDb();
   db.createDb();
   if (debug) debugger;
-  var numDocs = 500
+  var numDocs = 500;
   var docs = makeDocs(1,numDocs + 1);
   db.bulkSave(docs);
   var summate = function(N) {return (N+1)*N/2;};
 
   var map = function (doc) {
       emit(doc.integer, doc.integer);
-      emit(doc.integer, doc.integer)};
+      emit(doc.integer, doc.integer);
+  };
   var reduce = function (keys, values) { return sum(values); };
   var result = db.query(map, reduce);
   T(result.rows[0].value == 2*summate(numDocs));
@@ -69,7 +70,7 @@ couchTests.reduce = function(debug) {
       T(db.info().doc_count == ((i - 1) * 10 * 11) + ((j + 1) * 11));
     }
 
-    map = function (doc) {emit(doc.keys, 1)};
+    map = function (doc) { emit(doc.keys, 1); };
     reduce = function (keys, values) { return sum(values); };
 
     var results = db.query(map, reduce, {group:true});
@@ -99,6 +100,21 @@ couchTests.reduce = function(debug) {
     T(equals(results.rows[4], {key:["d","a"],value:10*i}));
     T(equals(results.rows[5], {key:["d","b"],value:10*i}));
     T(equals(results.rows[6], {key:["d","c"],value:10*i}));
+
+    // endkey test with inclusive_end=true
+    var results = db.query(map, reduce, {group_level:2,endkey:["d"],inclusive_end:true});
+    T(equals(results.rows[0], {key:["a"],value:20*i}));
+    T(equals(results.rows[1], {key:["a","b"],value:40*i}));
+    T(equals(results.rows[2], {key:["a","c"],value:10*i}));
+    T(equals(results.rows[3], {key:["d"],value:10*i}));
+    TEquals(4, results.rows.length);
+
+    // endkey test with inclusive_end=false
+    var results = db.query(map, reduce, {group_level:2,endkey:["d"],inclusive_end:false});
+    T(equals(results.rows[0], {key:["a"],value:20*i}));
+    T(equals(results.rows[1], {key:["a","b"],value:40*i}));
+    T(equals(results.rows[2], {key:["a","c"],value:10*i}));
+    TEquals(3, results.rows.length);
   }
 
   // now test out more complex reductions that need to use the combine option.
@@ -107,7 +123,7 @@ couchTests.reduce = function(debug) {
   db.createDb();
 
 
-  var map = function (doc) {emit(doc.val, doc.val)};
+  var map = function (doc) { emit(doc.val, doc.val); };
   var reduceCombine = function (keys, values, rereduce) {
       // This computes the standard deviation of the mapped results
       var stdDeviation=0.0;
