@@ -12,6 +12,20 @@
 
 couchTests.replication = function(debug) {
   if (debug) debugger;
+
+  function waitForSeq(sourceDb, targetDb) {
+    var targetSeq,
+        sourceSeq = sourceDb.info().update_seq,
+        t0 = new Date(),
+        t1,
+        ms = 3000;
+
+    do {
+      targetSeq = targetDb.info().update_seq;
+      t1 = new Date();
+    } while (((t1 - t0) <= ms) && targetSeq < sourceSeq);
+  }
+
   var host = CouchDB.host;
   var dbPairs = [
     {source:"test_suite_db_a",
@@ -22,14 +36,14 @@ couchTests.replication = function(debug) {
       target:"test_suite_db_b"},
     {source:"http://" + host + "/test_suite_db_a",
       target:"http://" + host + "/test_suite_db_b"}
-  ]
+  ];
   var dbA = new CouchDB("test_suite_db_a", {"X-Couch-Full-Commit":"false"});
   var dbB = new CouchDB("test_suite_db_b", {"X-Couch-Full-Commit":"false"});
   var numDocs = 10;
   var xhr;
   for (var testPair = 0; testPair < dbPairs.length; testPair++) {
-    var A = dbPairs[testPair].source
-    var B = dbPairs[testPair].target
+    var A = dbPairs[testPair].source;
+    var B = dbPairs[testPair].target;
 
     dbA.deleteDb();
     dbA.createDb();
@@ -41,7 +55,7 @@ couchTests.replication = function(debug) {
       test_template: new function () {
         this.init = function(dbA, dbB) {
           // before anything has happened
-        }
+        };
         this.afterAB1 = function(dbA, dbB) {
           // called after replicating src=A  tgt=B first time.
         };
@@ -165,20 +179,20 @@ couchTests.replication = function(debug) {
         this.afterAB1 = function(dbA, dbB) {
           var xhr = CouchDB.request("GET",
             "/test_suite_db_a/bin_doc/foo%2Bbar.txt");
-          T(xhr.responseText == "This is a base64 encoded text")
+          T(xhr.responseText == "This is a base64 encoded text");
 
           xhr = CouchDB.request("GET",
             "/test_suite_db_b/bin_doc/foo%2Bbar.txt");
-          T(xhr.responseText == "This is a base64 encoded text")
+          T(xhr.responseText == "This is a base64 encoded text");
 
           // and the design-doc
           xhr = CouchDB.request("GET",
             "/test_suite_db_a/_design/with_bin/foo%2Bbar.txt");
-          T(xhr.responseText == "This is a base64 encoded text")
+          T(xhr.responseText == "This is a base64 encoded text");
 
           xhr = CouchDB.request("GET",
             "/test_suite_db_b/_design/with_bin/foo%2Bbar.txt");
-          T(xhr.responseText == "This is a base64 encoded text")
+          T(xhr.responseText == "This is a base64 encoded text");
         };
       },
 
@@ -209,8 +223,8 @@ couchTests.replication = function(debug) {
           var docB = dbB.open("foo", {conflicts: true, deleted_conflicts: true});
 
           // We should have no conflicts this time
-          T(docA._conflicts === undefined)
-          T(docB._conflicts === undefined);
+          T(typeof docA._conflicts === "undefined");
+          T(typeof docB._conflicts === "undefined");
 
           // They show up as deleted conflicts instead
           T(docA._deleted_conflicts[0] == docB._deleted_conflicts[0]);
@@ -229,7 +243,7 @@ couchTests.replication = function(debug) {
 
     var seqA = result.source_last_seq;
     T(0 == result.history[0].start_last_seq);
-    T(result.history[1] === undefined)
+    T(typeof result.history[1] === "undefined");
 
     for(test in repTests) {
       if(repTests[test].afterAB1) repTests[test].afterAB1(dbA, dbB);
@@ -239,7 +253,7 @@ couchTests.replication = function(debug) {
 
     var seqB = result.source_last_seq;
     T(0 == result.history[0].start_last_seq);
-    T(result.history[1] === undefined)
+    T(typeof result.history[1] === "undefined");
 
     for(test in repTests) {
       if(repTests[test].afterBA1) repTests[test].afterBA1(dbA, dbB);
@@ -252,7 +266,7 @@ couchTests.replication = function(debug) {
 
     T(seqA < result2.source_last_seq);
     T(seqA == result2.history[0].start_last_seq);
-    T(result2.history[1].end_last_seq == seqA)
+    T(result2.history[1].end_last_seq == seqA);
 
     seqA = result2.source_last_seq;
 
@@ -260,11 +274,11 @@ couchTests.replication = function(debug) {
       if(repTests[test].afterAB2) repTests[test].afterAB2(dbA, dbB);
     }
 
-    result = CouchDB.replicate(B, A)
+    result = CouchDB.replicate(B, A);
 
     T(seqB < result.source_last_seq);
     T(seqB == result.history[0].start_last_seq);
-    T(result.history[1].end_last_seq == seqB)
+    T(result.history[1].end_last_seq == seqB);
 
     seqB = result.source_last_seq;
 
@@ -306,21 +320,21 @@ couchTests.replication = function(debug) {
   var continuousResult = CouchDB.replicate(dbA.name, "test_suite_db_b", {
     body: {"continuous": true}
   });
-  T(continuousResult.ok)
-  T(continuousResult._local_id)
+  T(continuousResult.ok);
+  T(continuousResult._local_id);
 
   var cancelResult = CouchDB.replicate(dbA.name, "test_suite_db_b", {
-    body: {"cancel": true}
+    body: {"continuous":true, "cancel": true}
   });
-  T(cancelResult.ok)
-  T(continuousResult._local_id == cancelResult._local_id)
+  T(cancelResult.ok);
+  T(continuousResult._local_id == cancelResult._local_id);
 
   try {
    var cancelResult2 = CouchDB.replicate(dbA.name, "test_suite_db_b", {
-     body: {"cancel": true}
+     body: {"continuous":true, "cancel": true}
    });
   } catch (e) {
-    T(e.error == "not_found")
+    T(e.error == "not_found");
   }
   // test replication object option doc_ids
 
@@ -527,4 +541,286 @@ couchTests.replication = function(debug) {
     T(docFoo4 === null);
   }
 
+  // test for COUCHDB-868 - design docs' attachments not getting replicated
+  // when doing a pull replication with HTTP basic auth
+  dbA = new CouchDB("test_suite_db_a");
+  dbB = new CouchDB("test_suite_db_b");
+  var usersDb = new CouchDB("test_suite_auth");
+  var lorem = CouchDB.request(
+    "GET", "/_utils/script/test/lorem.txt").responseText;
+  var lorem_b64 = CouchDB.request(
+    "GET", "/_utils/script/test/lorem_b64.txt").responseText;
+
+  usersDb.deleteDb();
+  usersDb.createDb();
+  dbA.deleteDb();
+  dbA.createDb();
+  dbB.deleteDb();
+  dbB.createDb();
+
+  var atts_ddoc = {
+    _id: "_design/i_have_atts",
+    language: "javascript"
+  };
+  T(dbA.save(atts_ddoc).ok);
+
+  var rev = atts_ddoc._rev;
+  var att_1_name = "lorem.txt";
+  var att_2_name = "lorem.dat";
+  var xhr = CouchDB.request(
+    "PUT", "/" + dbA.name + "/" + atts_ddoc._id + "/" + att_1_name + "?rev=" + rev, {
+      headers: {"Content-Type": "text/plain;charset=utf-8"},
+      body: lorem
+  });
+  rev = JSON.parse(xhr.responseText).rev;
+  T(xhr.status === 201);
+  xhr = CouchDB.request(
+    "PUT", "/" + dbA.name + "/" + atts_ddoc._id + "/" + att_2_name + "?rev=" + rev, {
+      headers: {"Content-Type": "application/data"},
+      body: lorem_b64
+  });
+  T(xhr.status === 201);
+
+  var fdmananaUserDoc = CouchDB.prepareUserDoc({
+    name: "fdmanana",
+    roles: ["reader"]
+  }, "qwerty");
+  T(usersDb.save(fdmananaUserDoc).ok);
+
+  T(dbA.setSecObj({
+    admins: {
+      names: [],
+      roles: ["admin"]
+    },
+    readers: {
+      names: [],
+      roles: ["reader"]
+    }
+  }).ok);
+  T(dbB.setSecObj({
+    admins: {
+      names: ["fdmanana"],
+      roles: []
+    }
+  }).ok);
+
+  var server_config = [
+    {
+      section: "couch_httpd_auth",
+      key: "authentication_db",
+      value: usersDb.name
+    },
+    // to prevent admin party mode
+    {
+      section: "admins",
+      key: "joe",
+      value: "erlang"
+    }
+  ];
+
+  var test_fun = function() {
+    T(CouchDB.login("fdmanana", "qwerty").ok);
+    T(CouchDB.session().userCtx.name === "fdmanana");
+    T(CouchDB.session().userCtx.roles.indexOf("_admin") === -1);
+
+    var repResult = CouchDB.replicate(
+      "http://fdmanana:qwerty@" + host + "/" + dbA.name,
+      dbB.name
+    );
+    T(repResult.ok === true);
+    T(repResult.history instanceof Array);
+    T(repResult.history.length === 1);
+    T(repResult.history[0].docs_written === 1);
+    T(repResult.history[0].docs_read === 1);
+    T(repResult.history[0].doc_write_failures === 0);
+
+    var atts_ddoc_copy = dbB.open(atts_ddoc._id);
+    T(atts_ddoc_copy !== null);
+    T(typeof atts_ddoc_copy._attachments === "object");
+    T(atts_ddoc_copy._attachments !== null);
+    T(att_1_name in atts_ddoc_copy._attachments);
+    T(att_2_name in atts_ddoc_copy._attachments);
+
+    var xhr = CouchDB.request("GET", "/" + dbB.name + "/" + atts_ddoc._id + "/" + att_1_name);
+    T(xhr.status === 200);
+    T(xhr.responseText === lorem);
+
+    xhr = CouchDB.request("GET", "/" + dbB.name + "/" + atts_ddoc._id + "/" + att_2_name);
+    T(xhr.status === 200);
+    T(xhr.responseText === lorem_b64);
+
+    CouchDB.logout();
+    T(CouchDB.login("joe", "erlang").ok);
+    T(dbA.setSecObj({
+      admins: {
+        names: [],
+        roles: ["bar"]
+      },
+      readers: {
+        names: [],
+        roles: ["foo"]
+      }
+    }).ok);
+    T(dbB.deleteDb().ok === true);
+    T(dbB.createDb().ok === true);
+    T(dbB.setSecObj({
+      admins: {
+        names: ["fdmanana"],
+        roles: []
+      }
+    }).ok);
+    CouchDB.logout();
+
+    T(CouchDB.login("fdmanana", "qwerty").ok);
+    T(CouchDB.session().userCtx.name === "fdmanana");
+    T(CouchDB.session().userCtx.roles.indexOf("_admin") === -1);
+    try {
+      repResult = CouchDB.replicate(
+        "http://fdmanana:qwerty@" + host + "/" + dbA.name,
+        dbB.name
+      );
+      T(false, "replication should have failed");
+    } catch(x) {
+      T(x.error === "unauthorized");
+    }
+
+    atts_ddoc_copy = dbB.open(atts_ddoc._id);
+    T(atts_ddoc_copy === null);
+
+    CouchDB.logout();
+    T(CouchDB.login("joe", "erlang").ok);
+  };
+
+  run_on_modified_server(server_config, test_fun);
+
+  // COUCHDB-1093 - filtered and continuous _changes feed dies when the
+  // database is compacted
+  dbA = new CouchDB("test_suite_db_a");
+  dbB = new CouchDB("test_suite_db_b");
+
+  dbA.deleteDb();
+  dbA.createDb();
+  dbB.deleteDb();
+  dbB.createDb();
+
+  var docs = makeDocs(1, 10);
+  docs.push({
+    _id: "_design/foo",
+    language: "javascript",
+    filters: {
+      myfilter: (function(doc, req) { return true; }).toString()
+    }
+  });
+  dbA.bulkSave(docs).ok;
+
+  var repResult = CouchDB.replicate(
+    "http://" + host + "/" + dbA.name,
+    dbB.name,
+    {
+      body: {
+        continuous: true,
+        filter: "foo/myfilter"
+      }
+    }
+  );
+  TEquals(true, repResult.ok);
+  TEquals('string', typeof repResult._local_id);
+
+  var xhr = CouchDB.request("GET", "/_active_tasks");
+  var tasks = JSON.parse(xhr.responseText);
+
+  TEquals(true, dbA.compact().ok);
+  while (dbA.info().compact_running) {};
+
+  TEquals(true, dbA.save(makeDocs(30, 31)[0]).ok);
+  xhr = CouchDB.request("GET", "/_active_tasks");
+
+  var tasksAfter = JSON.parse(xhr.responseText);
+  TEquals(tasks.length, tasksAfter.length);
+  waitForSeq(dbA, dbB);
+  T(dbB.open("30") !== null);
+
+  repResult = CouchDB.replicate(
+    "http://" + host + "/" + dbA.name,
+    dbB.name,
+    {
+      body: {
+        continuous: true,
+        filter: "foo/myfilter",
+        cancel: true
+      }
+    }
+  );
+  TEquals(true, repResult.ok);
+  TEquals('string', typeof repResult._local_id);
+
+
+  // COUCHDB-885 - push replication of a doc with attachment causes a
+ //  conflict in the target.
+  dbA = new CouchDB("test_suite_db_a");
+  dbB = new CouchDB("test_suite_db_b");
+
+  dbA.deleteDb();
+  dbA.createDb();
+  dbB.deleteDb();
+  dbB.createDb();
+
+  var doc = {
+    _id: "doc1"
+  };
+  TEquals(true, dbA.save(doc).ok);
+
+  repResult = CouchDB.replicate(
+    dbA.name,
+    "http://" + host + "/" + dbB.name
+  );
+  TEquals(true, repResult.ok);
+  TEquals(true, repResult.history instanceof Array);
+  TEquals(1, repResult.history.length);
+  TEquals(1, repResult.history[0].docs_written);
+  TEquals(1, repResult.history[0].docs_read);
+  TEquals(0, repResult.history[0].doc_write_failures);
+
+  doc["_attachments"] = {
+    "hello.txt": {
+      "content_type": "text/plain",
+      "data": "aGVsbG8gd29ybGQ="  // base64:encode("hello world")
+    },
+    "foo.dat": {
+      "content_type": "not/compressible",
+      "data": "aSBhbSBub3QgZ3ppcGVk"  // base64:encode("i am not gziped")
+    }
+  };
+
+  TEquals(true, dbA.save(doc).ok);
+  repResult = CouchDB.replicate(
+    dbA.name,
+    "http://" + host + "/" + dbB.name
+  );
+  TEquals(true, repResult.ok);
+  TEquals(true, repResult.history instanceof Array);
+  TEquals(2, repResult.history.length);
+  TEquals(1, repResult.history[0].docs_written);
+  TEquals(1, repResult.history[0].docs_read);
+  TEquals(0, repResult.history[0].doc_write_failures);
+
+  var copy = dbB.open(doc._id, {
+    conflicts: true, deleted_conflicts: true, attachments: true,
+    att_encoding_info: true});
+  T(copy !== null);
+  TEquals("undefined", typeof copy._conflicts);
+  TEquals("undefined", typeof copy._deleted_conflicts);
+  TEquals("text/plain", copy._attachments["hello.txt"]["content_type"]);
+  TEquals("aGVsbG8gd29ybGQ=", copy._attachments["hello.txt"]["data"]);
+  TEquals("gzip", copy._attachments["hello.txt"]["encoding"]);
+  TEquals("not/compressible", copy._attachments["foo.dat"]["content_type"]);
+  TEquals("aSBhbSBub3QgZ3ppcGVk", copy._attachments["foo.dat"]["data"]);
+  TEquals("undefined", typeof copy._attachments["foo.dat"]["encoding"]);
+  // end of test for COUCHDB-885
+
+
+  // cleanup
+  dbA.deleteDb();
+  dbB.deleteDb();
+  usersDb.deleteDb();
 };
